@@ -1,6 +1,10 @@
-from enum import Enum, IntEnum
+import logging
+from enum import Enum, IntEnum, unique
+
+_LOGGER = logging.getLogger(__name__)
 
 
+@unique
 class ProductType(IntEnum):
     NONE = 0
     ACTUATOR = 1
@@ -9,9 +13,12 @@ class ProductType(IntEnum):
     CONTROLLABLE = 4
     GATEWAY = 5
     INFRASTRUCTURE_COMPONENT = 6
+    GROUP = 7
 
 
+@unique
 class GatewayType(IntEnum):
+    UNKNOWN = -1
     VIRTUAL_KIZBOX = 0
     KIZBOX_V1 = 2
     TAHOMA = 15
@@ -45,9 +52,21 @@ class GatewayType(IntEnum):
     NEXITY_RAIL_DIN = 74
     ELIOT = 77
     WISER = 88
+    TAHOMA_SWITCH = 98
+
+    @classmethod
+    def _missing_(cls, value):  # type: ignore
+        _LOGGER.warning(f"Unsupported value {value} has been returned for {cls}")
+        return cls.UNKNOWN
+
+    @property
+    def beautify_name(self) -> str:
+        return self.name.replace("_", " ").title()
 
 
+@unique
 class GatewaySubType(IntEnum):
+    UNKNOWN = -1
     TAHOMA_BASIC = 1
     TAHOMA_BASIC_PLUS = 2
     TAHOMA_PREMIUM = 3
@@ -64,9 +83,19 @@ class GatewaySubType(IntEnum):
     TAHOMA_PRO = 14
     TAHOMA_SECURITY_SHORT_CHANNEL = 15
     TAHOMA_SECURITY_PRO = 16
-    TAHOMA_BOX_C_IO = 12
+    # TAHOMA_BOX_C_IO = 12 That’s probably 17, but tahomalink.com says it’s 12
+
+    @classmethod
+    def _missing_(cls, value):  # type: ignore
+        _LOGGER.warning(f"Unsupported value {value} has been returned for {cls}")
+        return cls.UNKNOWN
+
+    @property
+    def beautify_name(self) -> str:
+        return self.name.replace("_", " ").title()
 
 
+@unique
 class DataType(IntEnum):
     NONE = 0
     INTEGER = 1
@@ -80,6 +109,7 @@ class DataType(IntEnum):
     JSON_OBJECT = 11
 
 
+@unique
 class ExecutionType(str, Enum):
     IMMEDIATE_EXECUTION = "Immediate execution"
     DELAYED_EXECUTION = "Delayed execution"
@@ -89,6 +119,7 @@ class ExecutionType(str, Enum):
     RAW_TRIGGER_GATEWAY = "Raw trigger (Gateway)"
 
 
+@unique
 class ExecutionState(str, Enum):
     INITIALIZED = "INITIALIZED"
     NOT_TRANSMITTED = "NOT_TRANSMITTED"
@@ -100,6 +131,7 @@ class ExecutionState(str, Enum):
     QUEUED_SERVER_SIDE = "QUEUED_SERVER_SIDE"
 
 
+@unique
 class ExecutionSubType(str, Enum):
     ACTION_GROUP = "ACTION_GROUP"
     ACTION_GROUP_SEQUENCE = "ACTION_GROUP_SEQUENCE"
@@ -115,7 +147,9 @@ class ExecutionSubType(str, Enum):
     TIME_TRIGGER = "TIME_TRIGGER"
 
 
+@unique
 class FailureType(IntEnum):
+    UNKNOWN = -1
     NO_FAILURE = 0
     NON_EXECUTING = 11
     ERROR_WHILE_EXECUTING = 12
@@ -315,8 +349,15 @@ class FailureType(IntEnum):
     GATEWAY_BUFFER_OVERFLOW = 10006
     UNKNOWN_DETAILED_ERROR = 10007
 
+    @classmethod
+    def _missing_(cls, value):  # type: ignore
+        _LOGGER.warning(f"Unsupported value {value} has been returned for {cls}")
+        return cls.UNKNOWN
 
+
+@unique
 class EventName(str, Enum):
+    UNKNOWN = "Unknown"
     ACTION_GROUP_CREATED = "ActionGroupCreatedEvent"
     ACTION_GROUP_DELETED = "ActionGroupDeletedEvent"
     ACTION_GROUP_UPDATED = "ActionGroupUpdatedEvent"
@@ -380,12 +421,14 @@ class EventName(str, Enum):
     REFRESH_ALL_DEVICES_STATES_COMPLETED = "RefreshAllDevicesStatesCompletedEvent"
     SETUP_JOB_COMPLETED = "SetupJobCompletedEvent"
     SETUP_JOB_FAILED = "SetupJobFailedEvent"
+    SETUP_TRIGGER_TRIGGERED = "SetupTriggerTriggeredEvent"
     SOMFY_PROTECT_GET_SITES_COMPLETED = "SomfyProtectGetSitesCompletedEvent"
     SONOS_GET_TOPOLOGY_SUCCESS = "SonosGetTopologySuccessEvent"
     TOKEN_CREATED = "TokenCreatedEvent"
     TOKEN_CREATION_FAILED = "TokenCreationFailedEvent"
     TOKEN_DELETION_FAILED = "TokenDeletionFailedEvent"
     TOKEN_REMOVED = "TokenRemovedEvent"
+    PUSH_SUBSCRIPTION_CREATED = "PushSubscriptionCreatedEvent"
     VALID_ADDRESS = "ValidAddressEvent"
     ZIGBEE_BIND_NETWORK_COMPLETED = "ZigbeeBindNetworkCompletedEvent"
     ZIGBEE_BIND_NETWORK_FAILED = "ZigbeeBindNetworkFailedEvent"
@@ -396,7 +439,13 @@ class EventName(str, Enum):
     ZIGBEE_LEAVE_NETWORK_FAILED = "ZigbeeLeaveNetworkFailedEvent"
     ZIGBEE_REFRESH_NETWORK_COMPLETED = "ZigbeeRefreshNetworkCompletedEvent"
 
+    @classmethod
+    def _missing_(cls, value):  # type: ignore
+        _LOGGER.warning(f"Unsupported value {value} has been returned for {cls}")
+        return cls.UNKNOWN
 
+
+@unique
 class UpdateBoxStatus(str, Enum):
     NOT_UPDATABLE = "NOT_UPDATABLE"
     READY_TO_UPDATE = "READY_TO_UPDATE"
@@ -407,12 +456,14 @@ class UpdateBoxStatus(str, Enum):
     UPDATING = "UPDATING"
 
 
+@unique
 class CommandMode(str, Enum):
     HIGH_PRIORITY = "highPriority"
     GEOLOCATED = "geolocated"
     INTERNAL = "internal"
 
 
+@unique
 class UiWidget(str, Enum):
     # From /reference/ui/widgets
     AIR_FLOW_SENSOR = "AirFlowSensor"
